# https://pytorch.org/hub/pytorch_vision_alexnet/
# https://pytorch.org/vision/main/models/generated/torchvision.models.alexnet.html
# https://discuss.pytorch.org/t/why-does-alexnet-in-torch-vision-use-average-pooling/84313
# https://discuss.pytorch.org/t/alexnet-input-size-224-or-227/41272/3

import numpy as np
import cv2 as cv
from torchvision import transforms
import torchvision.models as models
import torch

preprocess = transforms.Compose([
    transforms.ToTensor(),
    transforms.Resize(256),
    transforms.CenterCrop(224),   
    transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]),
])


alex_net = models.alexnet(weights='DEFAULT')

print(alex_net)

image = cv.imread("cute-shepherd-dog-posing-isolated-white-background.jpg")
image_rgb = cv.cvtColor(image, cv.COLOR_BGR2RGB)

input_tensor = preprocess(image_rgb)
#print(input_tensor.shape)

input_tensor = input_tensor.unsqueeze(0)
#print(input_tensor.shape)

alex_net.eval()
output = alex_net(input_tensor)
probabilities = torch.nn.functional.softmax(output[0], dim=0)
#print(probabilities)

with open("imagenet_classes.txt", "r") as f:
    categories = [s.strip() for s in f.readlines()]
# Show top categories per image
top5_prob, top5_catid = torch.topk(probabilities, 5)
for i in range(top5_prob.size(0)):
    print(categories[top5_catid[i]], top5_prob[i].item())









