# pip install opencv-contrib-python
# https://stackoverflow.com/questions/57427233/module-cv2-cv2-has-no-attribute-ximgproc

# http://www.huppelen.nl/publications/selectiveSearchDraft.pdf

import cv2

from torchvision import transforms
import torchvision.models as models
import torch

alex_net = models.mobilenet_v3_small(weights='DEFAULT').eval()

with open("imagenet_classes.txt", "r") as f:
    categories = [s.strip() for s in f.readlines()]

preprocess = transforms.Compose([
    transforms.ToTensor(),
    transforms.Resize(256),
    transforms.CenterCrop(224),   
    transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]),
])

cv2.namedWindow("result", 0)

one_img = cv2.imread("dog.jpg")
one_img_res = cv2.resize(one_img, None, fx = 0.7, fy = 0.7 ) 
one_img_blur = cv2.medianBlur(one_img_res, 7)
one_img_res_paint = one_img_res.copy()
s_search = cv2.ximgproc.segmentation.createSelectiveSearchSegmentation()
s_search.setBaseImage(one_img_blur)
#s_search.switchToSelectiveSearchQuality()  
#s_search.switchToSelectiveSearchFast()              
s_search.switchToSingleStrategy()

rects = s_search.process()
print(len(rects))
for i in range(0, len(rects)):
    x, y, w, h = rects[i]
    if h < 30 or w < 30: continue
    #if h/w < 0.8 or h/w > 1.2: continue
    cv2.rectangle(one_img_blur, (x, y), (x + w, y + h), (0, 255, 0), 3)

    #cropp = one_img_res[y:y+h, x:x+w]
    #cropp_rgb = cv2.cvtColor(cropp, cv2.COLOR_BGR2RGB)
    
    #cv2.imshow("cropp", cropp)
    #cv2.waitKey()     
    
    #input_tensor = preprocess(cropp_rgb) 
    #input_tensor = input_tensor.unsqueeze(0) 
    #output = alex_net(input_tensor) 
    #output = output.squeeze(0)
    #probabilities = torch.nn.functional.softmax(output)
    
    #print("output", probabilities)   
    
    #top_prob, top_catid = torch.topk(probabilities, 1)
    #print("top_prob", top_prob)
    #print([categories[c] for c in top_catid])
    #cat_name, cat_prob = categories[top_catid], top_prob.item()    

    #if cat_prob < 0.8: continue
    #cv2.rectangle(one_img_res_paint, (x, y), (x + w, y + h), (0, 255, 0), 3)    
    #cv2.putText(one_img_res_paint, f"{cat_name}:{cat_prob:.2f}", (x, y), cv2.FONT_HERSHEY_SIMPLEX, 0.8, (0, 0, 255), 3)

    #cv2.imshow("cropp", cropp)
    #cv2.waitKey() 
    
cv2.imshow("one_img_res", one_img_res)
cv2.imshow("one_img_blur", one_img_blur)
cv2.imshow("one_img_res_paint", one_img_res_paint)
cv2.waitKey()
